@@ -12,9 +12,9 @@ import boto3
 import botocore
 from botocore.config import Config
 from pytz import timezone
+from graphqlclient import GraphQLClient
 
 from .constant import (
-    BASKING_API_URL,
     TIMEOUT_MESSAGE
 )
 from .graphql_query import GraphqlQuery
@@ -22,6 +22,8 @@ from .insight import Insight
 from .location import Location
 from .occupancy import Occupancy
 from .organization import Organization
+from .utils import Utils
+
 
 EMAIL_REGEX = re.compile(r"[^@]+@[^@]+\.[^@]+")
 
@@ -30,26 +32,21 @@ class Basking:
     """ Class Basking
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, log_level='WARNING', **kwargs):
         # Setting the logger
-        self.basking_api_url = BASKING_API_URL
         self.log = logging.getLogger(self.__class__.__name__)
-        self.log.debug('Started Basking SDK in debugging level using %s', self.basking_api_url)
+        self.log.setLevel(log_level)
 
         # cognito
         self.auth_type = 'env_variables'
         config = Config(signature_version=botocore.UNSIGNED)
-        self._boto3_client = boto3.client('cognito-idp', os.getenv('BASKING_AWS_REGION', 'eu-central-1'), config=config)
-        #   occupancy object
+        self.boto3_client = boto3.client('cognito-idp', os.getenv('BASKING_AWS_REGION', 'eu-central-1'), config=config)
         self.occupancy = Occupancy(basking_obj=self)
-        #   location object
         self.location = Location(basking_obj=self)
-        #   organization object
         self.organization = Organization(basking_obj=self)
-        #   graphql_query object
         self.graphql_query = GraphqlQuery(basking_obj=self)
-        #   insight object
         self.insight = Insight(basking_obj=self)
+        self.utils = Utils(basking_obj=self)
 
     def date_obj_to_timestamp_ms(self, date_obj, tz_str):
         """
@@ -89,23 +86,6 @@ class Basking:
             return True
         return False
 
-    def _cognito_login_password_auth_(self):
-        """
-        internal method to authenticate the user.
-
-        :return: return response
-        """
-        response = self._boto3_client.initiate_auth(
-            AuthFlow='USER_PASSWORD_AUTH',
-            AuthParameters={
-                'USERNAME': os.environ['BASKING_USERNAME'],
-                'PASSWORD': os.environ['BASKING_USER_PASSWORD']
-            },
-            ClientId='7o5vqv7ukt9onolq1nvjt7m40e'  # add the dashboard auth.
-        )
-
-        return response
-
     def api_timeout_handler(self, api_data):
         """
         for handle api timeout error coming from graphql_query
@@ -121,17 +101,6 @@ class Basking:
                 self.log.error(api_data['message'])
                 return TIMEOUT_MESSAGE
 
-    def get_new_cognito_token_(self):
-        """
-        Internal method to get new cognito token
-
-        :return: return new token
-        """
-
-        # return cognito_login()
-        return self._cognito_login_password_auth_(
-        )['AuthenticationResult']['IdToken']
-
     def basking_handle_api_return_errors(self, api_return_data):
         """
         checks if there are errors in the return from the API
