@@ -4,66 +4,104 @@ Basking.io — Python SDK
 - Graphql Query Class: handles all graphql queries.
 """
 import logging
+import os
+import re
 import urllib
+from dataclasses import dataclass, field
+from datetime import datetime, timedelta
+from typing import Optional
 from urllib.error import HTTPError
+
 import backoff
 from graphqlclient import GraphQLClient
 
+from src.basking.constant import BASKING_API_URL
+
+
+@dataclass
+class Authentication:
+    AccessToken: str
+    ExpiresIn: int
+    TokenType: str
+    IdToken: str
+    RefreshToken: str = field(default=None)
+    expires: datetime = field(init=False)
+
+    def __str__(self):
+        return f"Auth token (expires: {self.expires})"
+
 
 class GraphqlQuery:
     """
     graphql_query class
     """
     _ggraphql_client = None
+    DATE_FORMAT = '%Y-%m-%dT%H:%M:%S.000Z'
 
     def __init__(self, basking_obj):
         self.basking = basking_obj
+        self.basking_api_url = BASKING_API_URL
         self.log = logging.getLogger(self.__class__.__name__)
         basking_log_level = logging.getLogger(self.basking.__class__.__name__).level
         self.log.setLevel(basking_log_level)
+        self.log.debug('Started Basking SDK in debugging level using %s', self.basking_api_url)
+        self._auth_client = os.getenv('BASKING_AUTH_CLIENT', '3ehjj0o7hel3dpcmr8uu1ncckd')
+        self._graphql_client = GraphQLClient(self.basking_api_url)
+        self._auth = self.authenticate()
 
-    @property
-    def _graphql_client(self):
-        try:
-            if not self._ggraphql_client:
-                if self.basking.auth_type == 'env_variables':
-                    self.token = self.basking.get_new_cognito_token_()
-                else:
-                    raise ValueError('auth_type invalid.')
-                self._ggraphql_client = GraphQLClient(self.basking.basking_api_url)
-                self._ggraphql_client.inject_token(self.token)
-        except (TypeError, AttributeError, ValueError):
-            self.log.error("GraphQLClient error")
-        return self._ggraphql_client
+    def _authenticate(self):
+        response = self.basking.boto3_client.initiate_auth(
+            AuthFlow='USER_PASSWORD_AUTH',
+            AuthParameters={
+                'USERNAME': os.environ['BASKING_USERNAME'],
+                'PASSWORD': os.environ['BASKING_USER_PASSWORD']
+            },
+            ClientId=self._auth_client,
+        )
+        return Authentication(**response['AuthenticationResult'])
 
-    @backoff.on_exception(backoff.expo,
-                          urllib.error.HTTPError,
-                          max_tries=8)
-    def graphql_executor(
-            self,
-            query,
-            variables
-    ):
-        """Implements backoff for graphql
-
-        :param query: Graphql query to be executed
-        :type query: str.
-        :param variables: Variables for query
-        :type variables: dict.
-
-        :return: query result
+    def _refresh_token(self):
+        _refresh_token = self._auth.RefreshToken
+        response = self.basking.boto3_client.initiate_auth(
+            AuthFlow='REFRESH_TOKEN_AUTH',
+            AuthParameters={"REFRESH_TOKEN": _refresh_token},
+            ClientId=self._auth_client,
+        )
+        return Authentication(**response['AuthenticationResult'])
 
+    def authenticate(self) -> Optional[Authentication]:
+        """
+         Authenticates with AWS Cognito. Sets token expiry to T-10s for safety
+         Will attempt refreshing the token first, if possible
+         Handling authentication errors is the callers responsibility
         """
         try:
-            graphql_executor_variable = self._graphql_client.execute(
-                query=query,
+            auth = self._refresh_token()
+        except:
+            auth = self._authenticate()
+        finally:
+            auth.expires = datetime.now() + timedelta(seconds=auth.ExpiresIn - 10)
+            self._graphql_client.inject_token(auth.IdToken)
+            return auth
+
+    @backoff.on_exception(backoff.expo, urllib.error.HTTPError, max_tries=8)
+    def graphql_executor(self, query: str, variables: dict) -> Optional[dict]:
+        def _clean(query):
+            return query.replace('\n', ' ').replace('  ', ' ').replace('   ', ' ')
+
+        try:
+            if self._auth.expires <= datetime.now():
+                self.authenticate()
+            query_name = re.findall(r'query (.+?)\(.*', query.strip())[0]
+            self.log.info(f"Executing {query_name}({variables})")
+            return self._graphql_client.execute(
+                query=_clean(query),
                 variables=variables
             )
-            return graphql_executor_variable
         except AttributeError:
-            self.log.error("get error while execute query")
+            self.log.error("cannot execute query", exc_info=True)
         except HTTPError:
-            return {"message": "Api Time out error"}
+            self.log.error('API server error', exc_info=True)
 
     @staticmethod
     def get_building_occupancy_stats_daily_graphql_query(building_id, start_date_str, end_date_str):
@@ -658,11 +696,7 @@ class GraphqlQuery:
         return query, variables
 
     @staticmethod
-    def get_insight_graphql_query(
-            building_id,
-            start_datetime,
-            end_datetime
-    ):
+    def get_insight_graphql_query(building_id: str, start_datetime: datetime, end_datetime: datetime):
         """get_insight_data graphql query
 
         :param building_id: building_id
@@ -710,9 +744,9 @@ class GraphqlQuery:
             }
             """
         variables = {
-            "id": str(building_id),  # "5a52e94",
-            "from": str(start_datetime),  # "2021-11-20T23:00:00.000Z",
-            "to": str(end_datetime)  # "2022-02-20T23:00:00.000Z"
+            "id": building_id,
+            "from": start_datetime.isoformat() + 'Z',
+            "to": end_datetime.isoformat() + 'Z',
         }
         return query, variables
 
